package com.github.itzilly.sbt.features

import com.github.itzilly.sbt.SkyBlockTweaks
import com.github.itzilly.sbt.renderer.RenderFuncs
import net.minecraft.client.Minecraft
import net.minecraft.init.Blocks
import net.minecraft.util.BlockPos
import net.minecraftforge.client.event.RenderWorldLastEvent
import net.minecraftforge.common.config.Configuration
import net.minecraftforge.fml.common.eventhandler.SubscribeEvent

enum class GemstoneColors(val color: Int) {
    RUBY(0xFF5555),     // Red
    JADE(0x00AA00),     // Lime
    AMBER(0xFFAA00),    // Orange
    SAPPHIRE(0x55FFFF), // Light Blue
    AMETHYST(0xAA00AA), // Purple
    TOPAZ(0xFFFF55),    // Yellow
    JASPER(0xFF55FF)    // Magenta
}
object GemstoneFinder {
    private var gemstoneClusters: ArrayList<GemstoneCluster> = ArrayList()
    private var gemstoneList: ArrayList<GemstoneBlock> = ArrayList()
    var radius = SkyBlockTweaks.config.getCategory(Configuration.CATEGORY_GENERAL)?.get("gemstonesearchradius")?.int?: 16

    fun clear() {
        gemstoneClusters.clear()
    }

    fun find() {
        val playerPos = Minecraft.getMinecraft().thePlayer.position

        val minBounds = BlockPos(
            playerPos.x - radius,
            maxOf(playerPos.y - radius, 31),
            playerPos.z - radius
        )

        val maxBounds = BlockPos(
            playerPos.x + radius,
            minOf(playerPos.y + radius, 188),
            playerPos.z + radius
        )
        val world = Minecraft.getMinecraft().thePlayer.worldObj

        val gemstonesMap = mutableMapOf<GemstoneColors, MutableList<BlockPos>>()

        for (chunkX in minBounds.x / 16..maxBounds.x / 16) {
            for (chunkZ in minBounds.z / 16..maxBounds.z / 16) {
                val chunk = world.getChunkFromChunkCoords(chunkX, chunkZ)

                for (x in 0..15) {
                    for (y in minBounds.y..maxBounds.y) {
                        for (z in 0..15) {
                            val worldX = chunk.xPosition * 16 + x
                            val worldZ = chunk.zPosition * 16 + z

                            if (worldX in minBounds.x..maxBounds.x && worldZ in minBounds.z..maxBounds.z) {
                                val blockPos = BlockPos(worldX, y, worldZ)
                                val blockState = chunk.getBlockState(blockPos)
                                val block = blockState.block
                                if (block == Blocks.air) continue
                                val meta = block.getMetaFromState(blockState)

                                val gemstoneColor = when {
                                    block === Blocks.stained_glass || block === Blocks.stained_glass_pane -> {
                                        when (meta) {
                                            1 -> GemstoneColors.AMBER  // Orange
                                            2 -> GemstoneColors.JASPER  // Magenta
                                            3 -> GemstoneColors.SAPPHIRE  // Light Blue
                                            4 -> GemstoneColors.TOPAZ  // Yellow
                                            5 -> GemstoneColors.JADE  // Lime
                                            14 -> GemstoneColors.RUBY  // Red
                                            10 -> GemstoneColors.AMETHYST  // Purple
                                            else -> null
                                        }
                                    }
                                    else -> null
                                }

                                if (gemstoneColor != null) {
                                    gemstonesMap.getOrPut(gemstoneColor) { mutableListOf() }.add(blockPos)
                                }
                            }
                        }
                    }
                }
            }
        }

        for ((color, positions) in gemstonesMap) {
            val clusters = clusterGemstones(positions)
            for (cluster in clusters) {
                gemstoneClusters.add(GemstoneCluster(cluster, color))
            }
        }
    }

    private fun clusterGemstones(positions: List<BlockPos>): List<List<BlockPos>> {
        val clusters = mutableListOf<List<BlockPos>>()
        val visited = mutableSetOf<BlockPos>()

        fun dfs(start: BlockPos, cluster: MutableList<BlockPos>) {
            val stack = ArrayDeque<BlockPos>()
            stack.add(start)

            while (stack.isNotEmpty()) {
                val pos = stack.removeLast()
                if (pos in visited) continue
                visited.add(pos)
                cluster.add(pos)

                // Check adjacent blocks in all six directions
                for (offset in listOf(BlockPos(1, 0, 0), BlockPos(-1, 0, 0), BlockPos(0, 1, 0), BlockPos(0, -1, 0), BlockPos(0, 0, 1), BlockPos(0, 0, -1))) {
                    val neighbor = pos.add(offset)
                    if (neighbor in positions && neighbor !in visited) {
                        stack.add(neighbor)
                    }
                }
            }
        }

        for (pos in positions) {
            if (pos !in visited) {
                val cluster = mutableListOf<BlockPos>()
                dfs(pos, cluster)
                clusters.add(cluster)
            }
        }

        return clusters
    }

    @SubscribeEvent
    fun renderBlockOverlay(event: RenderWorldLastEvent) {
        val player = Minecraft.getMinecraft().thePlayer

        for (cluster in gemstoneClusters) {
            RenderFuncs.drawClusterOutline(player, cluster.positions, event.partialTicks, cluster.color.color)
        }
    }

    data class GemstoneCluster(
        val positions: List<BlockPos>,
        val color: GemstoneColors
    )


    data class GemstoneBlock(
        val x: Int,
        val y: Int,
        val z: Int,
        val color: GemstoneColors,
        val maxX: Int = x,
        val maxY: Int = y,
        val maxZ: Int = z
    )
}